% Stablyap+.m4
% Lyapunov fn
.PS
include(HOMELIB_`'lib3D.m4)
#nclude(/u2/aplevich/words/ee446/book/bookmacs.m4)
winclude(book.dim,Stablyap)
#xnclude(tst1.dim,test)

gen_init
viewaz = 30
viewel = 18
setview(viewaz,viewel)

Origin: project(0,0,0)
# Components of view vector W
w1 = m4caz*m4cel
w2 = m4saz*m4cel
w3 = m4sel
# Shape factor of the ellipse on the xy plane
q = Cos(40)

# cost function
h = 0.5
c = 1
# The projected ellipse is (x/q)^2 + y^2 = c.
# The cost is v = c+h
define(`vs',``$2'*q*cos(`$1'),`$2'*sin(`$1')')
define(`vp',`vs(`$1',`$2'),0')
define(`vx',`sum3D(vp(`$1',`$2'),0,0,h+(`$2')^2)')

# The gradient of v is (2x/q, 2y, -1) and the straight line
# separating front and back is W^T * grad(v) = 0
# This line intersects the projected ellipse at x1,y1 and x2,y2
  ap = w2^2*q^2/w1^2+1
  bp = -w2*w3*q^2/w1^2
  cp = w3^2*q^2/4/w1^2-c
  m = sqrt(bp^2-4*ap*cp)
  y1 = (-bp+m)/ap/2 ; x1 = (w3-2*y1*w2)*q/2/w1
  y2 = (-bp-m)/ap/2 ; x2 = (w3-2*y2*w2)*q/2/w1
  t1 = atan2(y1,x1)
  t2 = atan2(y2,x2)
  theta1 = min(t1,t2)
  theta2 = max(t1,t2)

# tangent curve
  nT = 11
  for i = 0 to nT do {
    y = y1 + (y2-y1)/nT*i
    theta = atan2(y,(w3-2*y*w2)*q/2/w1)
    r = y/sin(theta)
    T[i]: project(vx(theta,r))
    }

# front and back parts of the top curve
  n = 12
  for i = 0 to n do {
    theta = theta1 + (theta2-theta1)/n*i
    F[i]: project(vx(theta,c))
    Fp[i]: project(vp(theta,c))
    }
  for i = 0 to n do {
    theta = theta2 + (theta1+twopi_-theta2)/n*i
    B[i]: project(vx(theta,c))
    Bp[i]: project(vp(theta,c))
    }

# trajectory
rotations = 1.55
nx = 7
thetas = 75*dtor_
thetaf = thetas - rotations*twopi_
rx = c*0.9
beta = exp(log(.5)/20)

define(`defX',` rx = `$5' ; np = np-1
  ts = `$1' ; tf = `$2'
  for i = 0 to `$3' do {
    tha = ts + (tf-ts)*i/(`$3')
    for thx = tha to -twopi_ by twopi_ do {}
    `$4'[i]: project(vx(thx,rx))
    Xp[np]: project(vp(thx,rx))
    np = np+1
    rx = beta*rx
    }')

np = 1
defX(thetas,theta1,nx,X1,rx)
defX(theta1,theta2-twopi_,nx,X2,rx/beta)
defX(theta2-twopi_,theta1-twopi_,nx,X3,rx/beta)
defX(theta1-twopi_,thetaf,5,X4,rx/beta)

# First draw the inside back
\psset{gradbegin=lightgray,gradend=darkgray,gradlines=1000}
\pscustom[fillstyle=gradient,gradmidpoint=0.7]{
  fitcurve(B,n)
  for i = 0 to nT do {TT[i]: T[nT-i] }
  fitcurve(TT,nT)
\relax}

# Centre axis
thinlines_
arrow from Origin to project(0,0,2)
s_box($v(\BX)$) ljust
line from F[0] to Fp[0]
line from F[n] to Fp[n]
thicklines_

# Now draw the outside front (%s added to avoid a pstricks bug)
\newgray{gray1}{0.9}%
\newgray{gray2}{0.4}%
#fitcurve(B,n) # Added to get around an apparent pstricks bug
\psset{gradbegin=gray1,gradend=gray2,gradlines=1000}
\pscustom[linewidth=0pt,fillstyle=gradient,gradmidpoint=0.99]{
  fitcurve(F,n)
  fitcurve(T,nT)
\relax}
  fitcurve(T,nT)
  fitcurve(F,n)
# Front and back projections on xy
  fitcurve(Fp,n)
  fitcurve(Bp,n)

# The trajectory in pieces, to allow for dashed parts
  fitcurve(X1,nx)
# \psset{dash=3pt 2pt}
  fitcurve(X2,nx,dotted)
  fitcurve(X3,nx)
  fitcurve(X4,3,dotted)
# \psset{dash=5pt 3pt}
  line <- from X4[4] left .1*(X4[4+1].x-X4[4-1].x) \
    down .1*(X4[4+1].y-X4[4-1].y)

# Projected trajectory
  np = np-2
  fitcurve(Xp,np-1)
  line <- from Xp[np] left .1*(Xp[np+1].x-Xp[np-1].x) \
    down .1*(Xp[np+1].y-Xp[np-1].y)
  s_box($\!\!\BX(t)$) at Xp[np] ljust

# Axes and vertical lines
thinlines_
  line from X1[0] to Xp[0]
arrow from Origin to project(1.5,0,0)
s_box($x_1$) rjust below
arrow from Origin to project(0,1.5,0)
s_box($\;x_2$) ljust
line dashed from project(0,0,h) to project(0,0,2)

s_box(`$\,0$') at Origin+(0,1 pt__) below
s_box($\Omega$) at project(0,0.9*c,0) above
s_box(`$\,v(\BX) = c$') at project(vp(100*dtor_,c)) above ljust
.PE
